#!/bin/sh 
# TCPRCALL \
exec wish "$0" "$1" "$@"

##============================================================================
## geoevent.tcl -- (C) Claus-Peter R\"uckemann, 1999
##
## version 0.100
##
## This module creates a (tiny graphical user) interface for handling a
## simple database of coordinates and event relations for exploiting the
## -geolog option of the Vectaport viewers (utmviewer, dlgviewer, vpfviewer)
## introduced Jan 1999.
## Otherwise the -geolog option is only thought for logging coordinates
## actually visited. So the relation is only oneway.
##
##
## USAGE AND DESCRIPTION:
##
## IN COMBINATION WITH A VIEWER:
##
##   Calling from the -geolog option appends the current coordinate
##   pair xxx,yyy to the following:
##
##         whateverviewer -geolog geoevent.tcl
##
##   If the coordinate pair exists in the database then the associated
##   event is executed in background. Some messages are given to STDOUT.
##   What argument it gets (numbers, strings...) isn't important as long
##   as the apropriate string also occurs in the left column of the database.
##
## STANDALONE FOR TESTING OR PREPARING A SIMPLE DATABASE OF EVENTS:
##
##   You may test this standalone with a database (geoevent.dat)
##   e.g. containing:
##
##    {9,9} {xv}
##    {9,8} {xv -expand 2}
##    {8,8} {xv -expand 0.5}
##
##   with the following commands:
##
##         geoevent.tcl 8,1    -> no event associated, no GUI starts
##         geoevent.tcl 8,8    -> xv -expand 0.5 starts, no GUI starts
##         geoevent.tcl 8,1 xv -> [if there would be an event associated it
##                                would be started *and*] 
##                                if there is a 2nd arg the GUI always starts
##                                with the given values inserted into the
##                                fields waiting for pressing a button in
##                                order to:
##                                a) dismiss
##                                b) edit DB with $EDITOR
##                                c) add shown value pair to the database
##                                d) test-only the currently shown event
##
##   Button c) does not exit when pressed but clears the fields after having
##   added their contents to the database. The program therefore waits for
##   other events to add or another button pressed.
##   You will have to ensure by yourself that the database does not contain
##   crude stuff or redundant, empty or multiple entries for coordinate pairs
##   or events.
##
## ENVIRONMENT VARIABLES:
##
##   The following environment variables can be used for basic configuration:
##
##   GEOEVENT_DATPATH   The path the database file used. [default PWD]
##   GEOEVENT_DAT       The name of the database file. [default geoevent.dat]
##   EDITOR             What editor to use for editing the database.
##
##
## Tested on Linux.
## Tested to be compatible with various Tcl/Tk based windowing shells
## (wish4.2, wish 8.0, tixwish4.1.7.6, vtk, ...)
## under FVWM2 and KDE (and on AIX under CDE).
##
##                                 -------
##
## This file may be distributed with utmviewer/dlgviewer/vpfviewer
## under the terms of the GNU public license (GPL).
##
##                                 -------
##
## History:
##
## 19990130 CPR Started with version 0.100
## 19990201 CPR Added all the functionality :-)
##
##
## Bugs:
##
## No real error handling.
## No database functions or built in database editor.
## No validity checking.
## No quiet option.
##
## At least it is quite compact.
##
##
## Feel free to send me a mail if you made any improvements.
## ruckema@uni-muenster.de
##============================================================================

#BCMT-------------------------------------------------------------------------
###EN \gisigmodule{geoevent}
#ECMT-------------------------------------------------------------------------


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Location of the event database.}
#ECMT-------------------------------------------------------------------------
if [catch {
  set geoevent_datpath   $env(GEOEVENT_DAT)
} tmpcatch ] {
               set geoevent_dat "geoevent.dat"
               puts "No environment variable GEOEVENT_DAT,\
                     using: $geoevent_dat"
             }

if [catch {
  set geoevent_datpath   $env(GEOEVENT_DATPATH)
} tmpcatch ] {
               set geoevent_datpath "."
               puts "No environment variable GEOEVENT_DATPATH,\
                     using PWD: $geoevent_datpath"
             }


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Your personal EDITOR.}
#ECMT-------------------------------------------------------------------------
if [catch {
  set geoevent_editor   $env(EDITOR)
} tmpcatch ] {
        set geoevent_editor "vi"
        puts "No environment variable EDITOR, using vi"
             }


#BCMT-------------------------------------------------------------------------
###EN \section{settings}
#ECMT-------------------------------------------------------------------------
set text_dismiss     "Dismiss"
set text_edit        "Edit DB"
set text_add         "Add"
set text_testevent   "Test event"
set text_dismiss_bg  orange


#BCMT-------------------------------------------------------------------------
###EN \section{wm stuff}
#ECMT-------------------------------------------------------------------------
wm title . "Event Handler for Vectaport's Viewers"
wm minsize . 500 300
wm maxsize . 600 350
wm geometry . +150+350
wm resizable . yes no


#BCMT-------------------------------------------------------------------------
###EN \section{button bindings}
#ECMT-------------------------------------------------------------------------
bind . <Escape> exit_this


#BCMT-------------------------------------------------------------------------
###EN \gisigproc{CommandField}
###EN
###EN \begin{procarg}
###EN \procitem{name} label name
###EN \procitem{label} label text to be used on the canvas
###EN \procitem{width} width of the text on the canvas
###EN \procitem{command} command/procedure to execute, empty command is {}
###EN \procitem{args} arguments to the eval
###EN \end{procarg}
#ECMT-------------------------------------------------------------------------
proc CommandField { name label width command args } {
	frame $name
	label $name.label -text $label -width $width -anchor w
	eval {entry $name.entry -relief sunken -width 100} $args
	pack $name.label -side left -padx 1
	pack $name.entry -side right -fill x -expand true -padx 5 -pady 3
	bind $name.entry <Return> $command
	return $name.entry
}


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Fields}
###EN 
###EN Call the fields to be created, assign variables to the fields.
#ECMT-------------------------------------------------------------------------
CommandField .coord Coordinates 11 {} \
        -textvariable varcoord
CommandField .event Event 11 {} \
	-textvariable varevent


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Place the fields.}
###EN 
###EN And optionally design arangement.
#ECMT-------------------------------------------------------------------------
pack .coord .event


#BCMT-------------------------------------------------------------------------
###EN \section{Buttons}
#ECMT-------------------------------------------------------------------------
button .buttonbar_dismiss -text "$text_dismiss" -background $text_dismiss_bg \
   -width 9 \
   -command "exit"

button .buttonbar_edit -text "$text_edit" -width 9 \
   -command "geoevent_edit"

button .buttonbar_add -text "$text_add" -width 9 \
   -command "geoevent_add"

button .buttonbar_testevent -text "$text_testevent" -width 9 \
   -command "geoevent_testevent"


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Place the buttons.}
###EN 
###EN And design arangement.
#ECMT-------------------------------------------------------------------------

pack .buttonbar_testevent .buttonbar_add .buttonbar_edit .buttonbar_dismiss \
   -side right -padx 6 -pady 5


#BCMT-------------------------------------------------------------------------
###EN \gisigproc{geoevent\_testevent}
###EN 
###EN Test the event shown in the GUI event field.
#ECMT-------------------------------------------------------------------------
proc geoevent_testevent { } {

global varcoord varevent

exec "$varevent" &

}


#BCMT-------------------------------------------------------------------------
###EN \gisigproc{geoevent\_add}
###EN 
###EN Add the coordinate/event pair shown in the GUI fields to the database.
#ECMT-------------------------------------------------------------------------
proc geoevent_add { } {

global varcoord varevent geoevent_datpath geoevent_dat
set F [open $geoevent_datpath/$geoevent_dat a+]

puts $F "{$varcoord} {$varevent}"
close $F

# Clear when added event.
.coord.entry delete 0 end
.event.entry delete 0 end

#exit

}

#BCMT-------------------------------------------------------------------------
###EN \gisigproc{geoevent\_edit}
###EN 
###EN Edit the databse with your personal editor.
#ECMT-------------------------------------------------------------------------
proc geoevent_edit { } {

global varcoord varevent geoevent_datpath geoevent_dat geoevent_editor

exec xterm -e $geoevent_editor $geoevent_datpath/$geoevent_dat &

}


#BCMT-------------------------------------------------------------------------
###EN \gisigproc{exit\_this}
###EN
###EN Exits the application.
#ECMT-------------------------------------------------------------------------
proc exit_this {} {
    exit
}

#BCMT-------------------------------------------------------------------------
###EN \section{The main parts.}
#ECMT-------------------------------------------------------------------------


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Fill in current parameters.}
#ECMT-------------------------------------------------------------------------
.coord.entry       insert end [lindex $argv 1]
.event.entry       insert end [lindex $argv 2]


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Read the database into coord\_list}
#ECMT-------------------------------------------------------------------------
set F_IN [open $geoevent_datpath/$geoevent_dat r+]

while {[gets $F_IN event_entry] >= 0} {
  lappend coord_list_init $event_entry
}

set coord_list $coord_list_init

# Debugging only.
#puts $coord_list


#BCMT-------------------------------------------------------------------------
###EN \gisigsnip{Handle what's given.}
#ECMT-------------------------------------------------------------------------
if {[lindex $argv 1] != ""} {

  puts "Current args given: [lindex $argv 1] [lindex $argv 2]"

      # [lindex list index]
      set current_entry \
        [lindex $coord_list [lsearch -regexp $coord_list [lindex $argv 1] ] ]
      set current_event \
        [lindex $current_entry 1 ]

      puts "Entry: $current_entry  Event: $current_event"

      # Don't need GUI when database entry exists and no 2nd arg.
      if {[lindex $argv 2] != ""} {
      } else {
        destroy .
      }

      # So not to execute a string ;-)
      eval exec $current_event &

} else {
  
  puts "No args given!"

}

##EOF:
